# -*- coding: utf-8 -*-

import codecs
import os

import test


def read_vp_qtableview(file_name):
    f = codecs.open(file_name, "r", "utf8")
    s = f.read()
    f.close
    res = []
    for row in s.split("\n"):
        if len(row) == 0:
            continue
        column_values = column_values_string_to_column_values_array(row)
        res.append(column_values)
    return res


def column_values_string_to_column_values_array(column_values_string):
    return column_values_string.split("\t")


def column_values_array_to_column_values_string(column_values_array):
    for i in range(len(column_values_array)):
        column_values_array[i] = column_values_array[i].replace("\t", "\\t")
    return "\t".join(column_values_array)


def test_vp_qtableview_some_rows(vp_file_name, qtv, verification_title=None, create_vp_file_if_missing=True):
    if verification_title is None:
        test.startSection("Verification: %s" % os.path.basename(vp_file_name), vp_file_name)
    else:
        test.startSection("Verification: %s: %s" % (verification_title, os.path.basename(vp_file_name)), vp_file_name)

    if not os.path.exists(vp_file_name):
        dir_name = os.path.dirname(vp_file_name)
        if not os.path.exists(dir_name):
            os.makedirs(dir_name)
        open(vp_file_name, "w").close()

    expected = read_vp_qtableview(vp_file_name)
    actual = get_data_qtableview(qtv)
    failed = False
    try:
        if len(expected) == 0:
            failed = True
            test.fail("Expected rows data set is empty; must contain at least one row")
            return False

        skip_unmatched_rows_marker = ["*"]
        skip_unmatched_actual_rows = False
        current_actual_row_index = 0
        test.startSection("Comparison Details")
        try:
            exp_row = None
            for i in range(len(expected)):
                exp_row = expected[i]
                exp_row_str = column_values_array_to_column_values_string(exp_row)
                #test.log("Looking at-1: %s" % exp_row_str)

                # Check if end of actual reached:
                if current_actual_row_index == len(actual):
                    for i2 in range(i, len(expected)):
                        test.fail("Expected row #%s not found: '%s'" % (i2+1, exp_row_str))
                    return False

                if exp_row == skip_unmatched_rows_marker:
                    skip_unmatched_actual_rows = True
                    continue
    
                #test.log("Looking at-2: %s" % exp_row_str)

                while True: # ?
                    # Check if end of actual reached:
                    if current_actual_row_index == len(actual):
                        break
                    act_row = actual[current_actual_row_index]
                    if act_row == exp_row:
                        test.passes("Expected row #%s matches actual row #%s: Expected: '%s'" % (i+1, current_actual_row_index+1, exp_row_str))
                        current_actual_row_index += 1
                        skip_unmatched_actual_rows = False
                        break
        
                    elif skip_unmatched_actual_rows:
                        current_actual_row_index += 1
                        continue
        
                    else:
                        failed = True
                        test.fail("Expected row #%s not found: Expected: '%s'" % (i+1, exp_row_str), "Expected: '%s'" % exp_row_str)
                        return False
        finally:
            test.endSection()
    finally:
        if failed:
            write_vp_qtableview_fail_files(actual, vp_file_name)
        test.endSection()


def test_vp_qtableview_exact(vp_file_name, qtv, verification_title=None, create_vp_file_if_missing=True):
    if verification_title is None:
        test.startSection("Verification: %s" % os.path.basename(vp_file_name), vp_file_name)
    else:
        test.startSection("Verification: %s: %s" % (verification_title, os.path.basename(vp_file_name)), vp_file_name)

    if not os.path.exists(vp_file_name):
        dir_name = os.path.dirname(vp_file_name)
        if not os.path.exists(dir_name):
            os.makedirs(dir_name)
        open(vp_file_name, "w").close()

    expected = read_vp_qtableview(vp_file_name)
    actual = get_data_qtableview(qtv)
    failed = False
    try:
        if len(actual) != len(expected):
            failed = True
            test.fail("Different row count: len(actual): %s, len(expected): %s" % (len(actual), len(expected)))
            return False

        test.startSection("Comparison Details")
        try:
            for i in range(len(expected)):
                exp_row = expected[i]
                act_row = actual[i]
                if act_row == exp_row:
                    test.passes("Row #%s matches: '%s'" % (i+1, column_values_array_to_column_values_string(act_row)))
                else:
                    failed = True
                    test.fail("Row #%s mismatch: Expected: '%s', Actual: '%s'" % (i+1, column_values_array_to_column_values_string(exp_row), column_values_array_to_column_values_string(act_row)), "Expected: '%s'" % column_values_array_to_column_values_string(exp_row))
            return not failed
        finally:
            test.endSection()
    finally:
        if failed:
            write_vp_qtableview_fail_files(actual, vp_file_name)
        test.endSection()


def write_vp_qtableview(actual, vp_file_name):
    f = codecs.open(vp_file_name, "w", "utf8")
    for l in actual:
        f.write("%s" % column_values_array_to_column_values_string(l))
        f.write("\n")
    f.close()


def write_vp_qtableview_fail_files(actual, vp_file_name):
    dn = os.path.dirname(vp_file_name) + "/failed"
    if not os.path.exists(dn):
        os.makedirs(dn)

    fn = os.path.join(dn, "%s.%s" % (os.path.basename(vp_file_name), "vp_fail_txt"))
    f = codecs.open(fn, "w", "utf8")
    f.write("%s" % vp_file_name)
    f.close()

    # Attach to results:
    try:
        test.attachFile(fn, "QTableView verification results")
        os.remove(fn)
    except:
        test.log("QTableView verification results meta data created; %s" % fn)

    fn = os.path.join(dn, "%s.%s" % (os.path.basename(vp_file_name), "vp_fail_txt_data"))
    write_vp_qtableview(actual, fn)

    # Attach to results:
    try:
        test.attachFile(fn, "QTableView verification results data")
        os.remove(fn)
    except:
        test.log("QTableView verification results created; %s" % fn)


def get_data_qtableview(qtv):
    res = []
    m = qtv.model()
    col_values = []
    for row in range(m.rowCount()):
        for column in range(m.columnCount()):
            col_values.append("%s" % m.index(row, column).text)
        res.append(col_values)
        col_values = []
    return res
